﻿namespace EyeSoft.Nhibernate.Test.Domain
{
	using System;
	using System.Collections.Generic;
	using System.Linq;

	public class Customer
	{
		protected Customer()
		{
			Addresses = new List<Address>();
		}

		public virtual Guid Id { get; set; }

		public virtual string Name { get; set; }

		public virtual Fiscal Fiscal { get; set; }

		public virtual IList<Address> Addresses { get; protected set; }

		public static Customer Create(Guid id, string name, string code)
		{
			var customer = new Customer { Id = id, Name = name };
			var fiscal = Fiscal.Create(Guid.NewGuid(), code, customer);

			customer.Fiscal = fiscal;
			return customer;
		}

		public virtual void AddAddress(string street)
		{
			Addresses.Add(Address.Create(Guid.NewGuid(), street, this));
		}

		public override string ToString()
		{
			return Name;
		}

		public virtual Customer Clone()
		{
			var customer = new Customer { Id = Id, Name = Name };

			customer.Fiscal = Fiscal = (Fiscal == null) ? null : Fiscal.Clone(customer);

			var addressesList = Addresses.Select(x => x.Clone(customer)).ToList();

			customer.Addresses = addressesList;

			return customer;
		}
	}
}