﻿namespace EyeSoft.Nhibernate.Test
{
	using System;

	using EyeSoft.Data.Common;
	using EyeSoft.Nhibernate.Test.Data;
	using EyeSoft.Nhibernate.Test.Domain;

	using Microsoft.VisualStudio.TestTools.UnitTesting;

	using NHibernate.Tool.hbm2ddl;

	using SharpTestsEx;

	[TestClass]
	public class NhibernateMappingTest
	{
		[TestMethod]
		public void NhibernateSaveEntityWithCollectionLoadEntityRemoveItemFromCollection()
		{
			NhibernateHelper.DatabaseProvider().DropIfExistsAndCreate();

			var configuration = NhibernateHelper.Configuration();

			new SchemaExport(configuration).Create(true, true);

			using (var sessionFactory = configuration.BuildSessionFactory())
			{
				var id = Guid.NewGuid();

				using (var session = sessionFactory.OpenSession())
				{
					using (var transaction = session.BeginTransaction())
					{
						var customer = Customer.Create(id, "Bill", "CODE1");
						customer.AddAddress("Via Olmi");
						session.Merge(customer);

						transaction.Commit();
					}
				}

				using (var session = sessionFactory.OpenSession())
				{
					var customer = session.Load<Customer>(id).Clone();
					customer.Fiscal.Should().Not.Be.Null();
					customer.Addresses.Count.Should().Be.EqualTo(1);
				}

				using (var session = sessionFactory.OpenSession())
				{
					using (var transaction = session.BeginTransaction())
					{
						var customer = session.Get<Customer>(id).Clone();

						customer.Fiscal = null;
						customer.Addresses.RemoveAt(0);

						session.Merge(customer);

						transaction.Commit();
					}
				}

				using (var session = sessionFactory.OpenSession())
				{
					var customer = session.Get<Customer>(id).Clone();

					customer.Fiscal.Should().Be.Null();
					customer.Addresses.Should().Be.Empty();
				}
			}
		}
	}
}